import pandas as pd
from scipy.stats import zscore


def calculate_cronbach_alpha(file_path, columns):
    """
    Calculate Cronbach's alpha for specified columns in a CSV file.

    :param file_path: Path to the CSV file
    :param columns: List of columns to include in the calculation
    :return: Cronbach's alpha value
    """
    # Load the data from the CSV file
    data = pd.read_csv(file_path)

    # Identify the columns to be merged
    columns_to_merge = {
        'QSI_1': ['QSI_Smoke_1', 'QSI_Vape_1'],
        'QSI_2': ['QSI_Smoke_2', 'QSI_Vape_2'],
        'QSI_3': ['QSI_Smoke_3', 'QSI_Vape_3']
    }

    # Merge the columns
    for new_col, old_cols in columns_to_merge.items():
        data[new_col] = data[old_cols].sum(axis=1)

    columns_to_merge_mean = {
        'QSI': ['QSI_1', 'QSI_2', 'QSI_3']
    }

    # Merge the columns
    for new_col, old_cols in columns_to_merge_mean.items():
        data[new_col] = data[old_cols].sum(axis=1) / 3

    # Function to calculate Cronbach's alpha
    def cronbach_alpha(df):
        df_standardized = df.apply(zscore)
        n_items = df_standardized.shape[1]
        variances = df_standardized.var(axis=0, ddof=1)
        total_variance = df_standardized.sum(axis=1).var(ddof=1)

        return (n_items / (n_items - 1)) * (1 - variances.sum() / total_variance)

    # Select the columns for the specified items
    data_selected = data[columns]

    # Drop rows with missing values
    data_cleaned = data_selected.dropna()

    # Calculate Cronbach's alpha for the selected items
    alpha = cronbach_alpha(data_cleaned)

    return alpha


file_path = ''
columns = ['QSI_1', 'QSI_2', 'QSI_3']

if __name__ == '__main__':
    # Calculate the value for Cronbach's alpha
    alpha = calculate_cronbach_alpha(file_path, columns)
    print(f"Cronbach's alpha for {', '.join(columns)}: {alpha}")
