"""
Calculating correlation between each theme and a subset of user characteristics.

The raw data that is needed to run this:
- a csv file with data from the post-questionnaire.
- a csv file with the data from prolific
- a csv file with the coding of the responses

The correlation results are printed in the terminal.

Author: Yoan Naydenov
Date: June 2024
"""

import pandas as pd
from scipy.stats import spearmanr


def merge_csv_files(file_path1, file_path2, key, output_path=None):
    """
    Merge two CSV files based on a specified key and optionally save the merged data to a new file.

    Parameters:
        file_path1 (str): Path to the first CSV file.
        file_path2 (str): Path to the second CSV file.
        key (str): The column name to use as the merging key.
        output_path (str, optional): Path to save the merged CSV file. If None, the file is not saved.

    Returns:
        pd.DataFrame: The merged DataFrame.
    """
    # Load the datasets
    data1 = pd.read_csv(file_path1)
    data2 = pd.read_csv(file_path2)

    # Merge the datasets based on the provided key using an inner join
    merged_data = pd.merge(data1, data2, on=key, how='inner')

    # Save the merged dataset if an output path is provided
    if output_path:
        merged_data.to_csv(output_path, index=False)
        print(f"Merged data saved to {output_path}")

    return merged_data


def analyze_theme_correlations(data, path_codes, quantitative_columns):
    """
    Analyze correlations between themes and quantitative data in a dataset.

    Parameters:
        data (pd.DataFrame): The main dataset.
        path_codes (str): Path to the CSV file containing coded responses.
        quantitative_columns (list): List of columns with quantitative data for correlation analysis.

    Returns:
        pd.DataFrame: DataFrame containing Spearman correlation coefficients and p-values.
    """
    # Load the coded responses dataset
    codes_data = pd.read_csv(path_codes)

    # Merge the main dataset with the coded responses dataset based on 'rand_id'
    data = pd.merge(data, codes_data, on='rand_id', how='inner')

    # Identify the columns to be merged for quitter self-identity
    columns_to_merge = {
        'QSI_1': ['QSI_Smoke_1', 'QSI_Vape_1'],
        'QSI_2': ['QSI_Smoke_2', 'QSI_Vape_2'],
        'QSI_3': ['QSI_Smoke_3', 'QSI_Vape_3']
    }

    # Merge the columns
    for new_col, old_cols in columns_to_merge.items():
        data[new_col] = data[old_cols].sum(axis=1)

    columns_to_merge_mean = {
        'QSI': ['QSI_1', 'QSI_2', 'QSI_3']
    }

    # Merge the columns
    for new_col, old_cols in columns_to_merge_mean.items():
        data[new_col] = data[old_cols].sum(axis=1) / 3

    # Mapping codes to themes
    code_to_theme = {
        'trust': 1,
        'validating': 1,
        'emotional and understanding': 1,
        'support': 1,
        'real person': 1,
        'guidance': 2,
        'valuable': 2,
        'personalised': 2,
        'different perspective/expert': 2,
        'good/helpful advice': 2,
        'reminding': 3,
        'helpful': 3,
        'provoke thinking': 3,
        'engagement and interaction': 3,
        'effectiveness and reliability': 3,
        'more motivating': 4,
        'encouraging': 4,
        'accountability': 5
    }

    # Function to count the number of codes related to each theme in each response
    def count_codes_themes(row):
        codes = [row['code1'], row['code2']]
        theme_counts = {1: 0, 2: 0, 3: 0, 4: 0, 5: 0}
        for code in codes:
            if code in code_to_theme:
                theme_counts[code_to_theme[code]] = 1
        return theme_counts

    # Apply the function to create new columns for each theme count
    theme_counts_df = data.apply(count_codes_themes, axis=1, result_type='expand')
    theme_counts_df.columns = ['theme1', 'theme2', 'theme3', 'theme4', 'theme5']

    # Merge the theme counts with the original data
    data_with_counts = pd.concat([data, theme_counts_df], axis=1)

    # Select columns for correlation analysis

    correlation_data_counts = data_with_counts[['theme1', 'theme2', 'theme3', 'theme4',
                                                'theme5'] + quantitative_columns].dropna()

    # Calculate Spearman correlation coefficients and p-values
    def spearman_corr_pvals(df, col1, col2):
        corr, pval = spearmanr(df[col1], df[col2])
        return corr, pval

    # Calculate correlation coefficients and p-values for each pair of theme counts and quantitative data
    results = {}
    for theme in ['theme1', 'theme2', 'theme3', 'theme4', 'theme5']:
        for col in quantitative_columns:
            corr, pval = spearman_corr_pvals(correlation_data_counts, theme, col)
            results[(theme, col)] = (corr, pval)

    # Convert results to a DataFrame for easier viewing
    results_df = pd.DataFrame(results, index=['Spearman Correlation', 'p-value']).T

    return results_df


quantitative_columns1 = ['Age', 'P_engagement_R_1', 'P_accountability_R_1', 'P_effect_R_1', 'P_with_without_R_1']

quantitative_columns2 = ['QSI', 'Smoking_Freq', 'Weekly_Exercise']

# File path to the file with data from post questionnaires
file_path_post_questionnaire = ''
# File path to the file with data from prolific
file_path_prolific_data = ''
# File path to the file with codes of responses
file_path_codes = ''
if __name__ == '__main__':
    # Merge the post-questionnaire and prolific data
    merged_data = merge_csv_files(file_path_post_questionnaire, file_path_prolific_data, "rand_id", output_path=None)
    # Analyze theme correlations with the first set of quantitative columns
    correlation = analyze_theme_correlations(merged_data, file_path_codes,
                                             quantitative_columns1)
    print(correlation)
    # Analyze theme correlations with the second set of quantitative columns
    correlation = analyze_theme_correlations(merged_data, file_path_codes,
                                             quantitative_columns2)
    print(correlation)
